==> content_insights_report-1.0.1.tar.gz: content_insights_report/src/Controller/ContentInsightsReportController.php <==
<?php

namespace Drupal\content_insights_report\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\node\Entity\Node;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Database\Database;

class ContentInsightsReportController extends ControllerBase
{

  public function contentReport()
  {
    $config = \Drupal::config('content_insights_report.settings');
    $show_summary = $config->get('show_summary');
    $show_workflow_moderation_state_report = $config->get('show_workflow_moderation_state_report');
    $show_created_by_and_date = $config->get('show_created_by_and_date');
    $show_insights_report = $config->get('show_insights_report');
    $number_of_months = $config->get('number_of_months');
    $percentage_totals = $config->get('percentage_total');
    $sort_by_field = $config->get('sort_by_field');
    $sort_by_direction = $config->get('sort_by_direction');
    $show_print = $config->get('show_print');

    $report_data = [];

    // Fetch all content types
    $content_types = \Drupal::entityTypeManager()->getStorage('node_type')->loadMultiple();

    // Fetch Workflow content_moderation Template
    $moduleHandler = \Drupal::service('module_handler');
    $moderation_state = [];
    if ($moduleHandler->moduleExists('content_moderation')) {
      $workflows = \Drupal::entityTypeManager()->getStorage('workflow')->loadMultiple();
      // Loop through each workflow and get its states
      foreach ($workflows as $workflow) {
        // Get the states defined for this workflow
        $workflow_states = $workflow->getTypePlugin()->getStates();
        foreach ($workflow_states as $state) {
          // Published Moderation state
          $moderation_state[$workflow->id()]['name'] = $workflow->label();
          $moderation_state[$workflow->id()]['total_count'] = 0;
          $moderation_state[$workflow->id()]['status']['1']['name'] = 'Published';
          $moderation_state[$workflow->id()]['status']['1']['stage'][$state->id()]['name'] = $state->label();
          $moderation_state[$workflow->id()]['status']['1']['stage'][$state->id()]['count'] = 0;
          // Unpublished Moderation state
          $moderation_state[$workflow->id()]['status']['0']['name'] = 'Unpublished';
          $moderation_state[$workflow->id()]['status']['0']['stage'][$state->id()]['name'] = $state->label();
          $moderation_state[$workflow->id()]['status']['0']['stage'][$state->id()]['count'] = 0;
        }
      }
    }

    switch ($percentage_totals) {
      case "published_nodes":
        // Sum of all published nodes
        $percentage_totals_text = $this->t("Total percentage based on published nodes");
        break;
      case "all_nodes":
        // Sum of all nodes
        $percentage_totals_text = $this->t("Total percentage based on all nodes");
        break;
      case "unpublished_nodes":
        // Sum of all published nodes
        $percentage_totals_text = $this->t("Total percentage based on unpublished nodes");
        break;
    }

    // Prepare time intervals for # months
    $current_date = new DrupalDateTime();
    $months_ago = (new DrupalDateTime("first day of this month"))->modify('-' . $number_of_months . ' months');

    foreach ($content_types as $content_type) {
      $type_id = $content_type->id();
      $type = $content_type->getOriginalId();
      $type_name = $content_type->label();

      $type_description = $content_type->get('description');

      $content_type_data = [];

      // Content type stats: Published and Unpublished nodes count
      $published_nodes = $this->getNodeCount($content_type->id(), TRUE);
      $unpublished_nodes = $this->getNodeCount($content_type->id(), FALSE);

      $content_type_data['content_type_id'] = $type_id;
      $content_type_data['content_type_name'] = $type_name;
      $content_type_data['content_type_type'] = $type;
      $content_type_data['content_type_description'] = $type_description;

      switch ($percentage_totals) {
        case "published_nodes":
          // Sum of all published nodes
          $count_published_nodes = $this->getNodeCountAll(TRUE);
          $content_type_data['published_percent'] = $count_published_nodes ? ($published_nodes / $count_published_nodes) * 100 : 0;
          $content_type_data['percentage_totals'] = "* Published nodes";
          break;
        case "all_nodes":
          // Sum of all nodes
          $count_published_nodes = $this->getAllNodeCount();
          $content_type_data['published_percent'] = $count_published_nodes ? ($published_nodes / $this->getAllNodeCount()) * 100 : 0;
          $content_type_data['percentage_totals'] = "* All nodes";
          break;
        case "unpublished_nodes":
          // Sum of all published nodes
          $count_unpublished_nodes = $this->getNodeCountAll(FALSE);
          $content_type_data['published_percent'] = $count_unpublished_nodes ? ($unpublished_nodes / $count_unpublished_nodes) * 100 : 0;
          $content_type_data['percentage_totals'] = "* Unublished nodes";
          break;
      }

      $content_type_data['total_count'] = $published_nodes + $unpublished_nodes;
      $content_type_data['published_count'] = $published_nodes;
      $content_type_data['unpublished_count'] = $unpublished_nodes;

      // Fetch insights for # months
      $content_type_data['insights'] = $this->getInsightsCounts($content_type->id(), $months_ago, $current_date);

      // Moderation State Report
      if ($moduleHandler->moduleExists('content_moderation')) {
        $content_type_data['moderation_state'] = $this->getNodeSummaryPerModerationState($content_type->id(), $moderation_state);
      }

      // Add data to report
      $report_data[] = $content_type_data;
    }

    // Sort result by
    switch ($sort_by_field) {
      case "content_type":
        switch ($sort_by_direction) {
          case "asc":
            usort($report_data, function ($a, $b) {
              return strcmp($a["content_type_name"], $b["content_type_name"]);
            });
            break;
          case "desc":
            usort($report_data, function ($a, $b) {
              return strcmp($b["content_type_name"], $a["content_type_name"]);
            });
            break;
        }
        break;
      case "content_type_machine":
        switch ($sort_by_direction) {
          case "asc":
            usort($report_data, function ($a, $b) {
              return strcmp($a["content_type_type"], $b["content_type_type"]);
            });
            break;
          case "desc":
            usort($report_data, function ($a, $b) {
              return strcmp($b["content_type_type"], $a["content_type_type"]);
            });
            break;
        }
        break;
      case "total_percentage":
        switch ($sort_by_direction) {
          case "asc":
            usort($report_data, function ($a, $b) {
              return ($a['published_percent'] - $b['published_percent']) ? ($a['published_percent'] - $b['published_percent']) / abs($a['published_percent'] - $b['published_percent']) : 0; // Sort by year_month
            });
            break;
          case "desc":
            usort($report_data, function ($a, $b) {
              return ($b['published_percent'] - $a['published_percent']) ? ($b['published_percent'] - $a['published_percent']) / abs($b['published_percent'] - $a['published_percent']) : 0; // Sort by year_month
            });
            break;
        }
        break;
    }

    // Get current path for img
    $modulePath = \Drupal::service('module_handler')->getModule('content_insights_report')->getPath();
    return [
      '#theme' => 'content_insights_report',
      '#show_summary' => $show_summary,
      '#show_insights_report' => $show_insights_report,
      '#show_workflow_moderation_state_report' => $show_workflow_moderation_state_report,
      '#show_created_by_and_date' => $show_created_by_and_date,
      '#months' => $number_of_months,
      '#show_print' => $show_print,
      '#print_icon' => $modulePath . "/images/print-icon.png",
      '#report_data' => $report_data,
      '#percentage_totals' => $percentage_totals_text,
      '#attached' => [
        'library' => [
          'content_insights_report/content_insights_report',
        ],
      ],
    ];
  }

  private function getAllNodeCount()
  {
    $query = \Drupal::entityQuery('node')
      ->accessCheck(TRUE);
    return $query->count()->execute();
  }

  private function getNodeCountAll($is_published)
  {
    $query = \Drupal::entityQuery('node')
      ->condition('status', $is_published ? 1 : 0)
      ->accessCheck(TRUE);
    return $query->count()->execute();
  }

  private function getNodeCount($content_type_id, $is_published)
  {
    $query = \Drupal::entityQuery('node')
      ->condition('type', $content_type_id)
      ->condition('status', $is_published ? 1 : 0)
      ->accessCheck(TRUE);
    return $query->count()->execute();
  }

  private function getInsightsCounts($content_type_id, $start_date, $end_date)
  {
    $insights_counts = [];

    $config = \Drupal::config('content_insights_report.settings');
    $number_of_months = $config->get('number_of_months');
    $month_sort_by_direction = $config->get('month_sort_by_direction');

    for ($i = 1; $i <= $number_of_months; $i++) {
      $start_of_month = clone $start_date;
      $start_of_month = $start_of_month->modify('first day of ' . ($i === 0 ? 'this month' : "+$i months"));
      $end_of_month = clone $start_of_month;
      $end_of_month->modify('last day of this month');

      $insights_counts[] = [
        'yearmonth' => (int)$start_of_month->format('Ym'),
        'month' => $start_of_month->format('F Y'),
        'created_count' => $this->getNodeCreatedCount($content_type_id, $start_of_month, $end_of_month),
        'updated_count' => $this->getNodeUpdatedCount($content_type_id, $start_of_month, $end_of_month),
      ];
    }

    // Sort Month by
    switch ($month_sort_by_direction) {
      case "asc":
        usort($insights_counts, function ($a, $b) {
          return strcmp($a['yearmonth'], $b['yearmonth']); // Sort by year_month
        });
        break;
      case "desc":
        usort($insights_counts, function ($a, $b) {
          return strcmp($b['yearmonth'], $a['yearmonth']); // Sort by year_month
        });
        break;
    }

    return $this->flip_row_col_array($insights_counts);
  }

  private function flip_row_col_array($array)
  {
    $out = array();
    foreach ($array as $rowkey => $row) {
      foreach ($row as $colkey => $col) {
        $out[$colkey][$rowkey] = $col;
      }
    }
    return $out;
  }

  private function getNodeCreatedCount($content_type_id, $start_date, $end_date)
  {
    $query = \Drupal::entityQuery('node')
      ->condition('type', $content_type_id)
      ->condition('created', [$start_date->getTimestamp(), $end_date->getTimestamp()], 'BETWEEN')
      ->accessCheck(TRUE);
    return $query->count()->execute();
  }

  private function getNodeUpdatedCount($content_type_id, $start_date, $end_date)
  {
    $query = \Drupal::entityQuery('node')
      ->condition('type', $content_type_id)
      ->condition('changed', [$start_date->getTimestamp(), $end_date->getTimestamp()], 'BETWEEN')
      ->accessCheck(TRUE);
    return $query->count()->execute();
  }

  /**
   * Generates a summary report for nodes based on content type, status, and moderation state.
   *
   * @return array
   *   An array of counts grouped by content type, status, and moderation state.
   */
  private function getNodeSummaryPerModerationState( $content_type_id, $moderation_state)
  {

    $connection = Database::getConnection();

    // Query to count nodes grouped by content type and moderation state.
    $query = $connection->select('node_field_data', 'nfd');
    $query->fields('nfd', ['type','status']);
    $query->addField('cmsfd', 'workflow', 'workflow');
    $query->addField('cmsfd', 'moderation_state', 'moderation_state');
    $query->addExpression('COUNT(nfd.nid)', 'node_count');
    $query->leftJoin('content_moderation_state_field_data', 'cmsfd', 'nfd.nid = cmsfd.content_entity_id');
    $query->condition('nfd.type', $content_type_id);
    $query->isNotNull('cmsfd.moderation_state'); // Don't remove (website will load long and time out)
    $query->groupBy('nfd.type');
    $query->groupBy('nfd.status');
    $query->groupBy('cmsfd.workflow');
    $query->groupBy('cmsfd.moderation_state');

    $result = $query->execute();

    foreach ($result as $record) {
      $moderation_state[$record->workflow]['status'][$record->status]['stage'][$record->moderation_state ?: 'no_moderation_state']['count'] = $record->node_count;
      $moderation_state[$record->workflow]['total_count']++;
    }

    return $moderation_state;
  }
}

==> content_insights_report-1.0.4.tar.gz: content_insights_report/src/Controller/ContentInsightsReportController.php <==
<?php

namespace Drupal\content_insights_report\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Database;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;

/**
 * Defines a controller for the content_insights_report module.
 */
class ContentInsightsReportController extends ControllerBase {

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructs a content_insights_report object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager, ModuleHandlerInterface $module_handler) {
    $this->configFactory = $config_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public function contentReport() {
    $config = $this->configFactory->get('content_insights_report.settings');
    $show_summary = $config->get('show_summary');
    $show_workflow_moderation_state_report = $config->get('show_workflow_moderation_state_report');
    $show_created_by_and_date = $config->get('show_created_by_and_date');
    $show_insights_report = $config->get('show_insights_report');
    $number_of_months = $config->get('number_of_months');
    $percentage_totals = $config->get('percentage_total');
    $sort_by_field = $config->get('sort_by_field');
    $sort_by_direction = $config->get('sort_by_direction');
    $show_print = $config->get('show_print');

    $report_data = [];

    // Fetch all content types.
    $content_types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();

    // Fetch Workflow content_moderation Template.
    $moderation_state = [];
    if ($this->moduleHandler->moduleExists('content_moderation')) {
      $workflows = $this->entityTypeManager->getStorage('workflow')->loadMultiple();
      // Loop through each workflow and get its states.
      foreach ($workflows as $workflow) {
        // Get the states defined for this workflow.
        $workflow_states = $workflow->getTypePlugin()->getStates();
        foreach ($workflow_states as $state) {
          // Published Moderation state.
          $moderation_state[$workflow->id()]['name'] = $workflow->label();
          $moderation_state[$workflow->id()]['total_count'] = 0;
          $moderation_state[$workflow->id()]['status']['1']['name'] = 'Published';
          $moderation_state[$workflow->id()]['status']['1']['stage'][$state->id()]['name'] = $state->label();
          $moderation_state[$workflow->id()]['status']['1']['stage'][$state->id()]['count'] = 0;
          // Unpublished Moderation state.
          $moderation_state[$workflow->id()]['status']['0']['name'] = 'Unpublished';
          $moderation_state[$workflow->id()]['status']['0']['stage'][$state->id()]['name'] = $state->label();
          $moderation_state[$workflow->id()]['status']['0']['stage'][$state->id()]['count'] = 0;
        }
      }
    }

    switch ($percentage_totals) {
      case "published_nodes":
        // Sum of all published nodes.
        $percentage_totals_text = $this->t("Total percentage based on published nodes");
        break;

      case "all_nodes":
        // Sum of all nodes.
        $percentage_totals_text = $this->t("Total percentage based on all nodes");
        break;

      case "unpublished_nodes":
        // Sum of all published nodes.
        $percentage_totals_text = $this->t("Total percentage based on unpublished nodes");
        break;

    }

    // Prepare time intervals for # months.
    $current_date = new DrupalDateTime();
    $months_ago = (new DrupalDateTime("first day of this month"))->modify('-' . $number_of_months . ' months');

    foreach ($content_types as $content_type) {
      $type_id = $content_type->id();
      $type = $content_type->getOriginalId();
      $type_name = $content_type->label();
      $type_description = $content_type->get('description');
      $content_type_data = [];

      // Content type stats: Published and Unpublished nodes count.
      $published_nodes = $this->getNodeCount($content_type->id(), TRUE);
      $unpublished_nodes = $this->getNodeCount($content_type->id(), FALSE);

      $content_type_data['content_type_id'] = $type_id;
      $content_type_data['content_type_name'] = $type_name;
      $content_type_data['content_type_type'] = $type;
      $content_type_data['content_type_description'] = $type_description;

      switch ($percentage_totals) {
        case "published_nodes":
          // Sum of all published nodes.
          $count_published_nodes = $this->getNodeCountAll(TRUE);
          $content_type_data['published_percent'] = $count_published_nodes ? ($published_nodes / $count_published_nodes) * 100 : 0;
          $content_type_data['percentage_totals'] = "* Published nodes";
          break;

        case "all_nodes":
          // Sum of all nodes.
          $count_published_nodes = $this->getAllNodeCount();
          $content_type_data['published_percent'] = $count_published_nodes ? ($published_nodes / $this->getAllNodeCount()) * 100 : 0;
          $content_type_data['percentage_totals'] = "* All nodes";
          break;

        case "unpublished_nodes":
          // Sum of all published nodes.
          $count_unpublished_nodes = $this->getNodeCountAll(FALSE);
          $content_type_data['published_percent'] = $count_unpublished_nodes ? ($unpublished_nodes / $count_unpublished_nodes) * 100 : 0;
          $content_type_data['percentage_totals'] = "* Unublished nodes";
          break;

      }

      $content_type_data['total_count'] = $published_nodes + $unpublished_nodes;
      $content_type_data['published_count'] = $published_nodes;
      $content_type_data['unpublished_count'] = $unpublished_nodes;

      // Fetch insights for # months.
      $content_type_data['insights'] = $this->getInsightsCounts($content_type->id(), $months_ago, $current_date);

      // Moderation State Report.
      if ($this->moduleHandler->moduleExists('content_moderation')) {
        $content_type_data['moderation_state'] = $this->getNodeSummaryPerModerationState($content_type->id(), $moderation_state);
      }

      // Add data to report.
      $report_data[] = $content_type_data;
    }

    // Sort result by.
    switch ($sort_by_field) {
      case "content_type":
        switch ($sort_by_direction) {
          case "asc":
            usort($report_data, function ($a, $b) {
              return strcmp($a["content_type_name"], $b["content_type_name"]);
            });
            break;

          case "desc":
            usort($report_data, function ($a, $b) {
              return strcmp($b["content_type_name"], $a["content_type_name"]);
            });
            break;

        }
        break;

      case "content_type_machine":
        switch ($sort_by_direction) {
          case "asc":
            usort($report_data, function ($a, $b) {
              return strcmp($a["content_type_type"], $b["content_type_type"]);
            });
            break;

          case "desc":
            usort($report_data, function ($a, $b) {
              return strcmp($b["content_type_type"], $a["content_type_type"]);
            });
            break;

        }
        break;

      case "total_percentage":
        switch ($sort_by_direction) {
          case "asc":
            usort($report_data, function ($a, $b) {
              return ($a['published_percent'] - $b['published_percent']) ? ($a['published_percent'] - $b['published_percent']) / abs($a['published_percent'] - $b['published_percent']) : 0;
            });
            break;

          case "desc":
            usort($report_data, function ($a, $b) {
              return ($b['published_percent'] - $a['published_percent']) ? ($b['published_percent'] - $a['published_percent']) / abs($b['published_percent'] - $a['published_percent']) : 0;
            });
            break;

        }
        break;

    }

    // Get current path for img.
    $modulePath = $this->moduleHandler->getModule('content_insights_report')->getPath();
    return [
      '#theme' => 'content_insights_report',
      '#show_summary' => $show_summary,
      '#show_insights_report' => $show_insights_report,
      '#show_workflow_moderation_state_report' => $show_workflow_moderation_state_report,
      '#show_created_by_and_date' => $show_created_by_and_date,
      '#months' => $number_of_months,
      '#show_print' => $show_print,
      '#print_icon' => $modulePath . "/images/print-icon.png",
      '#report_data' => $report_data,
      '#percentage_totals' => $percentage_totals_text,
      '#attached' => [
        'library' => [
          'content_insights_report/content_insights_report',
        ],
      ],
    ];
  }

  /**
   * Count all number of nodes.
   *
   * @return int
   *   Count value of number of nodes.
   */
  private function getAllNodeCount() {
    $query = $this->entityTypeManager->getStorage('node')->getQuery();
    $query->accessCheck(TRUE);
    return $query->count()->execute();
  }

  /**
   * Count number of nodes based on status.
   *
   * @return int
   *   Count value of number of nodes.
   */
  private function getNodeCountAll($is_published) {
    $query = $this->entityTypeManager->getStorage('node')->getQuery();
    $query->condition('status', $is_published ? 1 : 0)
      ->accessCheck(TRUE);
    return $query->count()->execute();
  }

  /**
   * Count number of nodes based on status.
   *
   * @return int
   *   Count value of number of nodes.
   */
  private function getNodeCount($content_type_id, $is_published) {
    $query = $this->entityTypeManager->getStorage('node')->getQuery();
    $query->condition('type', $content_type_id)
      ->condition('status', $is_published ? 1 : 0)
      ->accessCheck(TRUE);
    return $query->count()->execute();
  }

  /**
   * Insights Counts values.
   *
   * @return array
   *   An array Insights.
   */
  private function getInsightsCounts($content_type_id, $start_date, $end_date) {
    $insights_counts = [];

    $config = $this->configFactory->get('content_insights_report.settings');
    $number_of_months = $config->get('number_of_months');
    $month_sort_by_direction = $config->get('month_sort_by_direction');

    for ($i = 1; $i <= $number_of_months; $i++) {
      $start_of_month = clone $start_date;
      $start_of_month = $start_of_month->modify('first day of ' . ($i === 0 ? 'this month' : "+$i months"));
      $end_of_month = clone $start_of_month;
      $end_of_month->modify('last day of this month');

      $insights_counts[] = [
        'yearmonth' => (int) $start_of_month->format('Ym'),
        'month' => $start_of_month->format('F Y'),
        'created_count' => $this->getNodeCreatedCount($content_type_id, $start_of_month, $end_of_month),
        'updated_count' => $this->getNodeUpdatedCount($content_type_id, $start_of_month, $end_of_month),
      ];
    }

    // Sort Month by.
    switch ($month_sort_by_direction) {
      case "asc":
        usort($insights_counts, function ($a, $b) {
          return strcmp($a['yearmonth'], $b['yearmonth']);
        });
        break;

      case "desc":
        usort($insights_counts, function ($a, $b) {
          return strcmp($b['yearmonth'], $a['yearmonth']);
        });
        break;

    }

    return $this->flipRowColArray($insights_counts);
  }

  /**
   * Flip array row to col.
   *
   * @return array
   *   An array of flipped values.
   */
  private function flipRowColArray($array) {
    $out = [];
    foreach ($array as $rowkey => $row) {
      foreach ($row as $colkey => $col) {
        $out[$colkey][$rowkey] = $col;
      }
    }
    return $out;
  }

  /**
   * Count number of nodes based on dates created.
   *
   * @return int
   *   Count value of number of nodes.
   */
  private function getNodeCreatedCount($content_type_id, $start_date, $end_date) {
    $query = $this->entityTypeManager->getStorage('node')->getQuery();
    $query->condition('type', $content_type_id)
      ->condition('created', [$start_date->getTimestamp(), $end_date->getTimestamp()], 'BETWEEN')
      ->accessCheck(TRUE);
    return $query->count()->execute();
  }

  /**
   * Count number of nodes based on dates updated.
   *
   * @return int
   *   Count value of number of nodes.
   */
  private function getNodeUpdatedCount($content_type_id, $start_date, $end_date) {
    $query = $this->entityTypeManager->getStorage('node')->getQuery();
    $query->condition('type', $content_type_id)
      ->condition('changed', [$start_date->getTimestamp(), $end_date->getTimestamp()], 'BETWEEN')
      ->accessCheck(TRUE);
    return $query->count()->execute();
  }

  /**
   * Generates based on content type, status, and moderation state.
   *
   * @return array
   *   An array of counts grouped by content type, status, and moderation state.
   */
  private function getNodeSummaryPerModerationState($content_type_id, $moderation_state) {

    $connection = Database::getConnection();

    // Query to count nodes grouped by content type and moderation state.
    $query = $connection->select('node_field_data', 'nfd');
    $query->fields('nfd', ['type', 'status']);
    $query->addField('cmsfd', 'workflow', 'workflow');
    $query->addField('cmsfd', 'moderation_state', 'moderation_state');
    $query->addExpression('COUNT(nfd.nid)', 'node_count');
    $query->leftJoin('content_moderation_state_field_data', 'cmsfd', 'nfd.nid = cmsfd.content_entity_id');
    $query->condition('nfd.type', $content_type_id);
    // Don't remove (website will load long and time out).
    $query->isNotNull('cmsfd.moderation_state');
    $query->groupBy('nfd.type');
    $query->groupBy('nfd.status');
    $query->groupBy('cmsfd.workflow');
    $query->groupBy('cmsfd.moderation_state');

    $result = $query->execute();

    foreach ($result as $record) {
      $moderation_state[$record->workflow]['status'][$record->status]['stage'][$record->moderation_state ?: 'no_moderation_state']['count'] = $record->node_count;
      $moderation_state[$record->workflow]['total_count']++;
    }

    return $moderation_state;
  }

}
